@@ -4,7 +4,7 @@ function(x, y = NULL, z = NULL, color = par("col"), pch = par("pch"),
      xlab = NULL, ylab = NULL, zlab = NULL, scale.y = 1, angle = 40,
      axis = TRUE, tick.marks = TRUE, label.tick.marks = TRUE,
      x.ticklabs = NULL, y.ticklabs = NULL, z.ticklabs = NULL,
-     y.margin.add = 0, grid = TRUE, box = TRUE, lab = par("lab"),
+     y.margin.add = 0, y.axis.offset = 1, grid = TRUE, box = TRUE, lab = par("lab"),
      lab.z = mean(lab[1:2]), type = "p", highlight.3d = FALSE,
      mar = c(5, 3, 4, 3) + 0.1, bg = par("bg"), col.axis = par("col.axis"),
      col.grid = "grey", col.lab = par("col.lab"), cex.symbols = par("cex"),
@@ -106,7 +106,7 @@ function(x, y = NULL, z = NULL, color = par("col"), pch = par("pch"),
 ### optim. axis scaling
     p.lab <- par("lab")
     ## Y
-    y.range <- range(dat$y[is.finite(dat$y)], ylim)
+    y.range <- y.range.fix <- range(dat$y[is.finite(dat$y)], ylim)
     y.prty <- pretty(y.range, n = lab[2],
         min.n = max(1, min(.5 * lab[2], p.lab[2])))
     y.scal <- round(diff(y.prty[1:2]), digits = 12)
@@ -116,7 +116,7 @@ function(x, y = NULL, z = NULL, color = par("col"), pch = par("pch"),
     if(!is.null(ylim)) y.max <- max(y.max, ceiling((ylim[2] - y.add) / y.scal))
 #    if(angle > 2) dat$y <- y.max - dat$y  ## turn y-values around
     ## X
-    x.range <- range(dat$x[is.finite(dat$x)], xlim)
+    x.range <- x.range.fix <- range(dat$x[is.finite(dat$x)], xlim)
     x.prty <- pretty(x.range, n = lab[1],
         min.n = max(1, min(.5 * lab[1], p.lab[1])))
     x.scal <- round(diff(x.prty[1:2]), digits = 12)
@@ -158,7 +158,7 @@ function(x, y = NULL, z = NULL, color = par("col"), pch = par("pch"),
     if(angle.2) {x1 <- x.min + yx.f * y.max; x2 <- x.max}
     else        {x1 <- x.min; x2 <- x.max + yx.f * y.max}
     plot.window(c(x1, x2), c(z.min, z.max + yz.f * y.max), asp = asp)
-    temp <- strwidth(format(rev(y.prty))[1], cex = cex.axis/par("cex"))
+    temp <- strwidth(paste0("M", format(rev(y.prty))[1]), cex = cex.axis * par("cex"), font = font.axis)
 
 ### lheight in usr units for numeric aspect is needed to locate
 ### side 2 and 4 axis annotation with fixes aspect.
@@ -206,7 +206,7 @@ function(x, y = NULL, z = NULL, color = par("col"), pch = par("pch"),
                 las <- par("las")
                 mytext <- function(labels, side, at, line = -0.5, ...)
                     mtext(text = labels, side = side, at = at, line = line,
-                          col=col.lab, cex=cex.axis, font=font.lab, ...)
+                          col=col.lab, cex=cex.axis*par("cex"), font=font.axis, ...)
                 ## X
                 if(is.null(x.ticklabs))
                     x.ticklabs <- format(i.x * x.scal)
@@ -246,16 +246,35 @@ function(x, y = NULL, z = NULL, color = par("col"), pch = par("pch"),
                     y.ticklabs <- rev(y.ticklabs)
                 text(i.y * yx.f + xx[1],
                      i.y * yz.f + z.min, y.ticklabs,
-                     pos=if(angle.1) 2 else 4, offset=1,
-                     col=col.lab, cex=cex.axis/par("cex"), font=font.lab)
+                     pos=if(angle.1) 2 else 4, offset=y.axis.offset,
+                     col=col.lab, cex=cex.axis, font=font.axis)
             }
         }
 
         ## axis and labels
 
+        ## determine position of labels
+        if(!is.na(asp)) {
+            if(angle.1) {
+                if(angle > 2) {
+                    linepad <- (x2 - usr[1])/lheight + 0.5
+                } else {
+                    linepad <- (x2 - usr[2])/lheight + 0.5
+                }
+            } else {
+                if(angle > 2) {
+                    linepad <- (usr[2] - x1)/lheight + 0.5
+                } else {
+                    linepad <- (usr[1] - x1)/lheight + 0.5
+                }
+            }
+        } else {
+            linepad = -0.5
+        }
+        
         mytext2 <- function(lab, side, line, at)
             mtext(lab, side = side, line = line, at = at, col = col.lab,
-                  cex = cex.lab, font = font.axis, las = 0)
+                  cex = cex.lab*par("cex"), font = font.lab, las = 0)
         ## X
         lines(c(x.min, x.max), c(z.min, z.min), col = col.axis, lty = lty.axis)
         if(!is.na(asp)) {
@@ -319,8 +338,8 @@ function(x, y = NULL, z = NULL, color = par("col"), pch = par("pch"),
 ### Return Function Object
     ob <- ls() ## remove all unused objects from the result's enviroment:
     rm(list = ob[!ob %in% c("angle", "mar", "usr", "x.scal", "y.scal", "z.scal", "yx.f",
-        "yz.f", "y.add", "z.min", "z.max", "x.min", "x.max", "y.max", 
-        "x.prty", "y.prty", "z.prty", "mem.par")])
+        "yz.f", "y.add", "z.min", "z.max", "x.min", "x.max", "y.max", "x.range.fix", "y.range.fix",
+        "xlabel", "ylabel", "zlabel", "x.prty", "y.prty", "z.prty", "mem.par")])
     rm(ob)
     invisible(list(
         xyz.convert = function(x, y=NULL, z=NULL) {
@@ -401,6 +420,60 @@ function(x, y = NULL, z = NULL, color = par("col"), pch = par("pch"),
             lines(c(x.min, x.min), c(z.min, z.max), ...)
             lines(c(x.min, x.max), c(z.min, z.min), ...)
         },
+        contour3d = function(f, x.count = 10, y.count = 10, type = "l", lty = "24", 
+            x.resolution = 50, y.resolution = 50, ...) {    
+            if(inherits(f, "lm")){
+                #orig.vars <- c(xlabel, ylabel, zlabel)
+                #orig.vars <- gsub(".*\\$", "", orig.vars)
+                vars <- all.vars(formula(f))
+            } else vars <- c("z", "x", "y")
+
+            #vars.ordering <- names(sort(sapply(vars, function(v) grep(v, orig.vars)))[1:2])
+            
+            # x vor y in Formel!
+            for(x1 in seq(x.range.fix[1], x.range.fix[2], length = x.count)){
+                d <- data.frame(x1, seq(y.range.fix[1], y.range.fix[2], length = y.resolution))
+                names(d) <- vars[-1]
+                if(inherits(f, "lm")){
+                    d[vars[1]] <- predict(f, newdata=d)      
+                } else d[vars[1]] <- f(d[[1]], d[[2]])
+                xyz <- xyz.coords(d)
+                if(angle > 2) { ## switch y and x axis to ensure righthand oriented coord.
+                    temp <- xyz$x; xyz$x <- xyz$y; xyz$y <- temp
+                }
+                y2 <- (xyz$y - y.add) / y.scal
+                x <- xyz$x / x.scal + yx.f * y2
+                y <- xyz$z / z.scal + yz.f * y2
+                mem.par <- par(mar = mar, usr = usr)
+                if(type == "h") {
+                    y2 <- z.min + yz.f * y2
+                    segments(x, y, x, y2, ...)
+                    points(x, y, type = "p", ...)
+                }
+                else points(x, y, type = type, lty = lty, ...)
+            }       
+            for(x2 in seq(y.range.fix[1], y.range.fix[2], length = y.count)){
+                d <- data.frame(seq(x.range.fix[1], x.range.fix[2], length = x.resolution), x2)
+                names(d) <- vars[-1]
+                if(inherits(f, "lm")){
+                    d[vars[1]] <- predict(f, newdata=d)      
+                } else d[vars[1]] <- f(d[[1]], d[[2]])
+                xyz <- xyz.coords(d)
+                if(angle > 2) { ## switch y and x axis to ensure righthand oriented coord.
+                    temp <- xyz$x; xyz$x <- xyz$y; xyz$y <- temp
+                }
+                y2 <- (xyz$y - y.add) / y.scal
+                x <- xyz$x / x.scal + yx.f * y2
+                y <- xyz$z / z.scal + yz.f * y2
+                mem.par <- par(mar = mar, usr = usr)
+                if(type == "h") {
+                    y2 <- z.min + yz.f * y2
+                    segments(x, y, x, y2, ...)
+                    points(x, y, type = "p", ...)
+                }
+                else points(x, y, type = type, lty = lty, ...)
+            }
+        },
         par.mar = mem.par
     ))
 }
